import { IJtd, IJtdRoot, JtdNode } from '@jtdc/types';
/**
 * Converts JTD and its dependencies to a map of nodes where key is `ref` and value is a parsed node.
 *
 * @param ref The ref of the root JTD.
 * @param jtdRoot The JTD to parse.
 *
 * @returns The map from ref to a parsed node.
 */
export declare function parseJtdRoot<M>(ref: string, jtdRoot: IJtdRoot<M>): Record<string, JtdNode<M>>;
/**
 * Converts JTD dependencies to a map of nodes where key is `ref` and value is a parsed node.
 *
 * @param jtdDefinitions The dictionary of ref-JTD pairs.
 *
 * @returns The map from ref to a parsed node.
 */
export declare function parseJtdDefinitions<M>(jtdDefinitions: Record<string, IJtd<M>>): Record<string, JtdNode<M>>;
/**
 * Converts JTD to a corresponding node.
 *
 * @param jtd The JTD to parse.
 *
 * @returns A parsed node.
 *
 * @see https://tools.ietf.org/html/rfc8927 RFC8927
 * @see https://jsontypedef.com/docs/jtd-in-5-minutes JTD in 5 minutes
 */
export declare function parseJtd<M>(jtd: IJtd<M>): JtdNode<M>;
