import { IJtdAnyNode, IJtdElementsNode, IJtdEnumNode, IJtdNullableNode, IJtdObjectNode, IJtdRefNode, IJtdTypeNode, IJtdUnionNode, IJtdValuesNode, JtdNode } from '@jtdc/types';
/**
 * Defines set of callbacks that {@link visitJtdNode} can invoke.
 */
export interface IJtdNodeVisitor<M> {
    any?: (node: IJtdAnyNode<M>) => void;
    ref?: (node: IJtdRefNode<M>) => void;
    nullable?: (node: IJtdNullableNode<M>, next: () => void) => void;
    type?: (node: IJtdTypeNode<M>) => void;
    enum?: (node: IJtdEnumNode<M>) => void;
    elements?: (node: IJtdElementsNode<M>, next: () => void) => void;
    values?: (node: IJtdValuesNode<M>, next: () => void) => void;
    object?: (node: IJtdObjectNode<M>, next: () => void) => void;
    property?: (propKey: string, propNode: JtdNode<M>, objectNode: IJtdObjectNode<M>, next: () => void) => void;
    optionalProperty?: (propKey: string, propNode: JtdNode<M>, objectNode: IJtdObjectNode<M>, next: () => void) => void;
    union?: (node: IJtdUnionNode<M>, next: () => void) => void;
    mapping?: (mappingKey: string, mappingNode: IJtdObjectNode<M>, unionNode: IJtdUnionNode<M>, next: () => void) => void;
}
/**
 * Invokes callbacks from `visitor` for each node tree under `node`.
 *
 * @param node The root of the JTD node tree.
 * @param visitor Callbacks that must be invoked for nodes under `node`.
 */
export declare function visitJtdNode<M>(node: JtdNode<M>, visitor: IJtdNodeVisitor<M>): void;
