/**
 * Invokes callbacks from `visitor` for each node tree under `node`.
 *
 * @param node The root of the JTD node tree.
 * @param visitor Callbacks that must be invoked for nodes under `node`.
 */
export function visitJtdNode(node, visitor) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    switch (node.nodeType) {
        case 0 /* ANY */:
            (_a = visitor.any) === null || _a === void 0 ? void 0 : _a.call(visitor, node);
            break;
        case 1 /* REF */:
            (_b = visitor.ref) === null || _b === void 0 ? void 0 : _b.call(visitor, node);
            break;
        case 2 /* NULLABLE */:
            (_c = visitor.nullable) === null || _c === void 0 ? void 0 : _c.call(visitor, node, function () { return visitJtdNode(node.valueNode, visitor); });
            break;
        case 3 /* TYPE */:
            (_d = visitor.type) === null || _d === void 0 ? void 0 : _d.call(visitor, node);
            break;
        case 4 /* ENUM */:
            (_e = visitor.enum) === null || _e === void 0 ? void 0 : _e.call(visitor, node);
            break;
        case 5 /* ELEMENTS */:
            (_f = visitor.elements) === null || _f === void 0 ? void 0 : _f.call(visitor, node, function () { return visitJtdNode(node.elementNode, visitor); });
            break;
        case 6 /* VALUES */:
            (_g = visitor.values) === null || _g === void 0 ? void 0 : _g.call(visitor, node, function () { return visitJtdNode(node.valueNode, visitor); });
            break;
        case 7 /* OBJECT */:
            (_h = visitor.object) === null || _h === void 0 ? void 0 : _h.call(visitor, node, function () { return visitJtdObjectNodeProperties(node, visitor); });
            break;
        case 8 /* UNION */:
            (_j = visitor.union) === null || _j === void 0 ? void 0 : _j.call(visitor, node, function () {
                var _a;
                var _loop_1 = function (mappingKey, mappingNode) {
                    (_a = visitor.mapping) === null || _a === void 0 ? void 0 : _a.call(visitor, mappingKey, mappingNode, node, function () { return visitJtdObjectNodeProperties(mappingNode, visitor); });
                };
                for (var _i = 0, _b = Object.entries(node.mapping); _i < _b.length; _i++) {
                    var _c = _b[_i], mappingKey = _c[0], mappingNode = _c[1];
                    _loop_1(mappingKey, mappingNode);
                }
            });
            break;
    }
}
function visitJtdObjectNodeProperties(node, visitor) {
    var _a, _b;
    var _loop_2 = function (propKey, propNode) {
        (_a = visitor.property) === null || _a === void 0 ? void 0 : _a.call(visitor, propKey, propNode, node, function () { return visitJtdNode(propNode, visitor); });
    };
    for (var _i = 0, _c = Object.entries(node.properties); _i < _c.length; _i++) {
        var _d = _c[_i], propKey = _d[0], propNode = _d[1];
        _loop_2(propKey, propNode);
    }
    var _loop_3 = function (propKey, propNode) {
        (_b = visitor.optionalProperty) === null || _b === void 0 ? void 0 : _b.call(visitor, propKey, propNode, node, function () { return visitJtdNode(propNode, visitor); });
    };
    for (var _e = 0, _f = Object.entries(node.optionalProperties); _e < _f.length; _e++) {
        var _g = _f[_e], propKey = _g[0], propNode = _g[1];
        _loop_3(propKey, propNode);
    }
}
