import { __assign } from "tslib";
import { compileTsTypes, tsTypesCompilerOptions } from './ts-types-compiler';
import { parseJtdDefinitions } from './jtd-ast';
import { createMap, die } from './misc';
import { compileValidators, validatorCompilerOptions } from './validators-compiler';
import { compileJsSource } from '@smikhalevski/codegen';
import { createJtdDialect, jtdDialectOptions } from '@jtdc/jtd-dialect';
/**
 * Compiles map of JTD definitions to a map of corresponding sources.
 *
 * @param jtdModules The map from module URI to JTD definitions map.
 * @param options Compiler options.
 */
export function compileTsModules(jtdModules, options) {
    var opts = __assign(__assign(__assign(__assign({}, validatorCompilerOptions), tsTypesCompilerOptions), jtdDialectOptions), options);
    var validatorsRendered = opts.validatorsRendered, _a = opts.dialectFactory, dialectFactory = _a === void 0 ? createJtdDialect : _a;
    var dialect = dialectFactory(opts);
    var tsModules = createMap();
    var modules = Object.entries(jtdModules).map(function (_a) {
        var uri = _a[0], jtdDefinitions = _a[1];
        var definitions = parseJtdDefinitions(jtdDefinitions);
        var exports = extractExports(definitions, opts);
        return { uri: uri, definitions: definitions, exports: exports };
    });
    var _loop_1 = function (module_1) {
        var imports = createMap();
        opts.resolveExternalRef = function (node) {
            var _a;
            for (var _i = 0, modules_2 = modules; _i < modules_2.length; _i++) {
                var module_2 = modules_2[_i];
                var exportedRef = module_2.exports[node.ref];
                if (exportedRef) {
                    var importedNames = imports[_a = module_2.uri] || (imports[_a] = new Set());
                    importedNames.add(exportedRef.typeName);
                    if (validatorsRendered) {
                        importedNames.add(exportedRef.validatorName);
                    }
                    return exportedRef.typeName;
                }
            }
            die('Unresolved reference: ' + node.ref);
        };
        var src = compileTsTypes(module_1.definitions, opts);
        for (var _b = 0, _c = Object.entries(imports); _b < _c.length; _b++) {
            var _d = _c[_b], uri = _d[0], importedNames = _d[1];
            src = "import{" + Array.from(importedNames).sort().join(',') + "}from" + JSON.stringify(uri) + ";" + src;
        }
        if (validatorsRendered && dialect != null) {
            src = compileJsSource(dialect.import()) + src;
            src += compileValidators(module_1.definitions, opts);
        }
        tsModules[module_1.uri] = src;
    };
    for (var _i = 0, modules_1 = modules; _i < modules_1.length; _i++) {
        var module_1 = modules_1[_i];
        _loop_1(module_1);
    }
    return tsModules;
}
function extractExports(definitions, options) {
    var exports = createMap();
    for (var _i = 0, _a = Object.entries(definitions); _i < _a.length; _i++) {
        var _b = _a[_i], ref = _b[0], node = _b[1];
        exports[ref] = {
            typeName: options.renameType(ref, node),
            validatorName: options.renameValidator(ref, node),
        };
    }
    return exports;
}
