import { __assign } from "tslib";
import { visitJtdNode } from './jtd-visitor';
import { compileJsSource, template as _ } from '@smikhalevski/codegen';
import { createJtdDialect } from '@jtdc/jtd-dialect';
/**
 * Compiles validators and type guards from the definitions.
 */
export function compileValidators(definitions, options) {
    var opt = __assign(__assign({}, validatorCompilerOptions), options);
    var typeGuardsRendered = opt.typeGuardsRendered, dialect = opt.dialect;
    var src = '';
    var _loop_1 = function (ref, node) {
        src += compileJsSource(dialect.validator(ref, node, function (ctx) { return compileValidatorBody(node, ctx, dialect); }));
        if (typeGuardsRendered) {
            src += compileJsSource(dialect.typeGuard(ref, node));
        }
    };
    for (var _i = 0, _a = Object.entries(definitions); _i < _a.length; _i++) {
        var _b = _a[_i], ref = _b[0], node = _b[1];
        _loop_1(ref, node);
    }
    return src;
}
function compileValidatorBody(node, ctx, dialect) {
    var frags = [];
    var createNext = function (next) { return function (nextCtx) {
        var prevFrags = frags;
        var prevCtx = ctx;
        frags = [];
        ctx = nextCtx;
        next();
        var nextFrag = _(frags);
        frags = prevFrags;
        ctx = prevCtx;
        return nextFrag;
    }; };
    visitJtdNode(node, {
        ref: function (node) {
            frags.push(dialect.ref(node, ctx));
        },
        nullable: function (node, next) {
            frags.push(dialect.nullable(node, ctx, createNext(next)));
        },
        type: function (node) {
            frags.push(dialect.type(node, ctx));
        },
        enum: function (node) {
            frags.push(dialect.enum(node, ctx));
        },
        elements: function (node, next) {
            frags.push(dialect.elements(node, ctx, createNext(next)));
        },
        values: function (node, next) {
            frags.push(dialect.values(node, ctx, createNext(next)));
        },
        object: function (node, next) {
            frags.push(dialect.object(node, ctx, createNext(next)));
        },
        property: function (propKey, propNode, objectNode, next) {
            frags.push(dialect.property(propKey, propNode, objectNode, ctx, createNext(next)));
        },
        optionalProperty: function (propKey, propNode, objectNode, next) {
            frags.push(dialect.optionalProperty(propKey, propNode, objectNode, ctx, createNext(next)));
        },
        union: function (node, next) {
            frags.push(dialect.union(node, ctx, createNext(next)));
        },
        mapping: function (mappingKey, mappingNode, unionNode, next) {
            frags.push(dialect.mapping(mappingKey, mappingNode, unionNode, ctx, createNext(next)));
        },
    });
    return _(frags);
}
export var validatorCompilerOptions = {
    typeGuardsRendered: false,
    dialect: createJtdDialect(),
};
